@@ -19,6 +19,7 @@
 #include "egg-toolbars-model.h"
 #include "eggmarshalers.h"
 
+#include <unistd.h>
 #include <string.h>
 #include <libxml/tree.h>
 #include <gdk/gdkproperty.h>
@@ -145,7 +146,7 @@ safe_save_xml (const char *xml_file, xmlDocPtr doc)
 	tmp_file = g_strconcat (xml_file, ".tmp", NULL);
 	old_file = g_strconcat (xml_file, ".old", NULL);
 
-	if (!xmlSaveFormatFile (tmp_file, doc, 1))
+	if (xmlSaveFormatFile (tmp_file, doc, 1) <= 0)
 	{
 		g_warning ("Failed to write XML data to %s", tmp_file);
 		goto failed;
@@ -325,9 +326,9 @@ impl_add_item (EggToolbarsModel    *t,
   EggToolbarsItem *item;
   int real_position;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
-  g_return_if_fail (id != NULL);
-  g_return_if_fail (type != NULL);
+  g_return_val_if_fail (IS_EGG_TOOLBARS_MODEL (t), FALSE);
+  g_return_val_if_fail (id != NULL, FALSE);
+  g_return_val_if_fail (type != NULL, FALSE);
 
   parent_node = g_node_nth_child (t->priv->toolbars, toolbar_position);
   item = toolbars_item_new (id, type, FALSE);
@@ -422,7 +423,7 @@ parse_toolbars (EggToolbarsModel *t,
     }
 }
 
-void
+gboolean
 egg_toolbars_model_load (EggToolbarsModel *t,
 			 const char *xml_file)
 {
@@ -432,12 +433,19 @@ egg_toolbars_model_load (EggToolbarsModel *t,
   g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
 
   doc = xmlParseFile (xml_file);
+  if (doc == NULL)
+  {
+    g_warning ("Failed to load XML data from %s", xml_file);
+    return FALSE;
+  }
   root = xmlDocGetRootElement (doc);
 
   t->priv->toolbars = g_node_new (NULL);
   parse_toolbars (t, root->children);
 
   xmlFreeDoc (doc);
+
+  return TRUE;
 }
 
 char *
