@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -51,10 +51,6 @@ impl_construct (EphyDialog *dialog,
 		const EphyDialogProperty *properties,
 		const char *file,
 		const char *name);
-
-static void
-impl_destruct (EphyDialog *dialog);
-
 static GtkWidget *
 impl_get_control (EphyDialog *dialog,
                   int property_id);
@@ -106,6 +102,7 @@ struct EphyDialogPrivate
 	PropertyInfo *props;
 	gboolean modal;
 	gboolean has_default_size;
+	gboolean disposing;
 	char *name;
 
 	int spin_item_id;
@@ -145,6 +142,19 @@ ephy_dialog_get_type (void)
         return ephy_dialog_type;
 }
 
+static void
+ephy_dialog_dispose (GObject *object)
+{
+	EphyDialog *dialog = EPHY_DIALOG (object);
+
+	if (dialog->priv->dialog)
+	{
+		dialog->priv->disposing = TRUE;
+		gtk_widget_destroy (dialog->priv->dialog);
+		dialog->priv->dialog = NULL;
+	}
+}
+
 static void
 ephy_dialog_class_init (EphyDialogClass *klass)
 {
@@ -153,6 +163,7 @@ ephy_dialog_class_init (EphyDialogClass *klass)
         parent_class = g_type_class_peek_parent (klass);
 
         object_class->finalize = ephy_dialog_finalize;
+	object_class->dispose = ephy_dialog_dispose;
 	object_class->set_property = ephy_dialog_set_property;
 	object_class->get_property = ephy_dialog_get_property;
 
@@ -161,7 +172,6 @@ ephy_dialog_class_init (EphyDialogClass *klass)
 	klass->get_value = impl_get_value;
 	klass->run = impl_run;
 	klass->show = impl_show;
-	klass->destruct = impl_destruct;
 
 	g_object_class_install_property (object_class,
                                          PROP_PARENT_WINDOW,
@@ -797,6 +807,7 @@ ephy_dialog_init (EphyDialog *dialog)
 	dialog->priv->name = NULL;
 	dialog->priv->initialized = FALSE;
 	dialog->priv->has_default_size = FALSE;
+	dialog->priv->disposing = FALSE;
 }
 
 static void
@@ -937,11 +948,6 @@ ephy_dialog_finalize (GObject *object)
 
         g_return_if_fail (dialog->priv != NULL);
 
-	if (dialog->priv->dialog)
-	{
-		ephy_dialog_destruct (dialog);
-	}
-
 	free_props (dialog->priv->props);
 
 	g_free (dialog->priv->name);
@@ -1064,23 +1070,18 @@ init_props (const EphyDialogProperty *properties, GladeXML *gxml)
 	return props;
 }
 
-static void
-dialog_destruction_notify (EphyDialog *dialog,
-                           GObject *where_the_object_was)
-{
-	dialog->priv->dialog = NULL;
-	ephy_dialog_destruct (dialog);
-	g_object_unref (dialog);
-}
-
 static void
 dialog_destroy_cb (GtkWidget *widget, EphyDialog *dialog)
 {
-	if (dialog->priv->props &&
-	    dialog->priv->dialog)
+	if (dialog->priv->props)
 	{
 		save_props (dialog->priv->props);
 	}
+
+	if (!dialog->priv->disposing)
+	{
+		g_object_unref (dialog);
+	}
 }
 
 static void
@@ -1104,29 +1105,12 @@ impl_construct (EphyDialog *dialog,
 		dialog->priv->props = init_props (properties, gxml);
 	}
 
-	g_signal_connect (dialog->priv->dialog,
-			  "destroy",
-			  G_CALLBACK(dialog_destroy_cb),
-			  dialog);
+	g_signal_connect_object (dialog->priv->dialog,
+			         "destroy",
+			         G_CALLBACK(dialog_destroy_cb),
+			         dialog, 0);
 
 	g_object_unref (gxml);
-
-	g_object_weak_ref (G_OBJECT(dialog->priv->dialog),
-			   (GWeakNotify)dialog_destruction_notify,
-			   dialog);
-}
-
-static void
-impl_destruct (EphyDialog *dialog)
-{
-	if (dialog->priv->dialog)
-	{
-		g_object_weak_unref (G_OBJECT(dialog->priv->dialog),
-				     (GWeakNotify)dialog_destruction_notify,
-				     dialog);
-		gtk_widget_destroy (dialog->priv->dialog);
-		dialog->priv->dialog = NULL;
-	}
 }
 
 static GtkWidget *
@@ -1196,7 +1180,7 @@ setup_default_size (EphyDialog *dialog)
 static gint
 impl_run (EphyDialog *dialog)
 {
-	setup_default_size (dialog);
+	ephy_dialog_show (dialog);
 
 	return gtk_dialog_run (GTK_DIALOG(dialog->priv->dialog));
 }
@@ -1245,13 +1229,6 @@ ephy_dialog_construct (EphyDialog *dialog,
         return klass->construct (dialog, properties, file, name);
 }
 
-void
-ephy_dialog_destruct (EphyDialog *dialog)
-{
-	EphyDialogClass *klass = EPHY_DIALOG_GET_CLASS (dialog);
-        klass->destruct (dialog);
-}
-
 gint
 ephy_dialog_run (EphyDialog *dialog)
 {
