@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.20 2003/08/08 17:33:34 chpe Exp $
+ *  $Id: ephy-node-view.c,v 1.26.2.1 2003/09/04 22:19:43 marco Exp $
  */
 
 #include <gtk/gtktreeselection.h>
@@ -68,6 +68,17 @@ struct EphyNodeViewPrivate
 	int editable_property;
 
 	int searchable_data_column;
+
+	gboolean drag_started;
+	int drag_button;
+	int drag_x;
+        int drag_y;
+	GtkTargetList *source_target_list;
+
+	gboolean drop_occurred;                                                                                          
+	gboolean have_drag_data;
+	GtkSelectionData *drag_data;
+	guint scroll_id;
 };
 
 enum
@@ -86,6 +97,8 @@ enum
 	PROP_FILTER
 };
 
+#define AUTO_SCROLL_MARGIN 20
+
 static GObjectClass *parent_class = NULL;
 
 static guint ephy_node_view_signals[LAST_SIGNAL] = { 0 };
@@ -202,6 +215,11 @@ ephy_node_view_finalize (GObject *object)
 	g_object_unref (G_OBJECT (view->priv->filtermodel));
 	g_object_unref (G_OBJECT (view->priv->nodemodel));
 
+	if (view->priv->source_target_list)
+	{
+		gtk_target_list_unref (view->priv->source_target_list);
+	}
+
 	g_free (view->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -213,6 +231,8 @@ get_node_from_path (EphyNodeView *view, GtkTreePath *path)
 	EphyNode *node;
 	GtkTreeIter iter, iter2;
 
+	if (path == NULL) return NULL;
+
 	gtk_tree_model_get_iter (view->priv->sortmodel, &iter, path);
 	gtk_tree_model_sort_convert_iter_to_child_iter
 		(GTK_TREE_MODEL_SORT (view->priv->sortmodel), &iter2, &iter);
@@ -223,103 +243,272 @@ get_node_from_path (EphyNodeView *view, GtkTreePath *path)
 	return node;
 }
 
-static gboolean
-drag_motion_cb (GtkWidget *widget,
-                GdkDragContext *context,
-                gint x,
-                gint y,
-                guint time,
-	        EphyNodeView *view)
+static void
+gtk_tree_view_vertical_autoscroll (GtkTreeView *tree_view)
 {
-	EphyNode *node;
-	GtkTreePath *path = NULL;
-	GtkTreeViewDropPosition pos;
-	gboolean res;
-	EphyNodeViewPriority priority;
-
-	g_signal_stop_emission_by_name (widget, "drag_motion");
-
-	res = gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
-					         x, y, &path, &pos);
-	if (!res) return TRUE;
-
-	node = get_node_from_path (view, path);
-
-	priority = ephy_node_get_property_int (node, view->priv->priority_prop_id);
-
-	if (priority != EPHY_NODE_VIEW_ALL_PRIORITY &&
-	    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY)
+	GdkRectangle visible_rect;
+	GtkAdjustment *vadjustment;
+	GdkWindow *window;
+	int y;
+	int offset;
+	float value;
+	
+	window = gtk_tree_view_get_bin_window (tree_view);
+	vadjustment = gtk_tree_view_get_vadjustment (tree_view);
+	
+	gdk_window_get_pointer (window, NULL, &y, NULL);
+	
+	y += vadjustment->value;
 
+	gtk_tree_view_get_visible_rect (tree_view, &visible_rect);
+	
+	offset = y - (visible_rect.y + 2 * AUTO_SCROLL_MARGIN);
+	if (offset > 0)
 	{
-		gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (widget), path,
-						 GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
-		gdk_drag_status (context, context->suggested_action, time);
+		offset = y - (visible_rect.y + visible_rect.height - 2 * AUTO_SCROLL_MARGIN);
+		if (offset < 0)
+		{
+			return;
+		}
+	}
+
+	value = CLAMP (vadjustment->value + offset, 0.0,
+		       vadjustment->upper - vadjustment->page_size);
+	gtk_adjustment_set_value (vadjustment, value);
+}
+
+static int
+scroll_timeout (gpointer data)
+{
+	GtkTreeView *tree_view = GTK_TREE_VIEW (data);
+	
+	gtk_tree_view_vertical_autoscroll (tree_view);
+
+	return TRUE;
+}
+
+static void
+remove_scroll_timeout (EphyNodeView *view)
+{
+	if (view->priv->scroll_id)
+	{
+		g_source_remove (view->priv->scroll_id);
+		view->priv->scroll_id = 0;
+	}
+}
+
+static void
+set_drag_dest_row (EphyNodeView *view,
+		   GtkTreePath *path)
+{
+	if (path)
+	{
+		gtk_tree_view_set_drag_dest_row
+			(GTK_TREE_VIEW (view),
+			 path,
+			 GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
 	}
 	else
 	{
-		gdk_drag_status (context, 0, time);
+		gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (view), 
+						 NULL, 
+						 0);
 	}
-
-	gtk_tree_path_free (path);
-
-	return TRUE;
 }
 
-static gboolean
-drag_drop_cb (GtkWidget *widget,
-              GdkDragContext *context,
-              gint x,
-              gint y,
-              guint time,
-	      EphyNodeView *view)
+static void
+clear_drag_dest_row (EphyNodeView *view)
+{
+	gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (view), NULL, 0);
+}
+
+static void
+get_drag_data (EphyNodeView *view,
+	       GdkDragContext *context, 
+	       guint32 time)
 {
 	GdkAtom target;
+	
+	target = gtk_drag_dest_find_target (GTK_WIDGET (view), 
+					    context, 
+					    NULL);
 
-	g_signal_stop_emission_by_name (widget, "drag_drop");
+	gtk_drag_get_data (GTK_WIDGET (view),
+			   context, target, time);
+}
 
-	target = gtk_drag_dest_find_target (widget, context,
-					    view->priv->drag_targets);
+static void
+free_drag_data (EphyNodeView *view)
+{
+	view->priv->have_drag_data = FALSE;
 
-	if (target != GDK_NONE)
+	if (view->priv->drag_data)
 	{
-		gtk_drag_get_data (widget, context, target, time);
+		gtk_selection_data_free (view->priv->drag_data);
+		view->priv->drag_data = NULL;
 	}
+}
+
+static gboolean
+drag_motion_cb (GtkWidget *widget,
+		GdkDragContext *context,
+		int x,
+		int y,
+		guint32 time,
+		EphyNodeView *view)
+{
+	EphyNode *node;
+	GdkAtom target;
+	GtkTreePath *path;
+	GtkTreeViewDropPosition pos;
+	guint action = 0;
+	int priority;
+
+	gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
+					   x, y, &path, &pos);
+	
+	if (!view->priv->have_drag_data)
+	{
+		get_drag_data (view, context, time);
+	}
+
+	target = gtk_drag_dest_find_target (widget, context, NULL);
+	node = get_node_from_path (view, path);
+
+	if (target != GDK_NONE && node != NULL)
+	{
+		priority = ephy_node_get_property_int
+				(node, view->priv->priority_prop_id);
+
+		if (priority != EPHY_NODE_VIEW_ALL_PRIORITY &&
+		    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY)
+		{
+			action = context->suggested_action;
+		}
+	}
+	
+	if (action)
+	{
+		set_drag_dest_row (view, path);
+	}
+	else
+	{
+		clear_drag_dest_row (view);
+	}
+	
+	if (path)
+	{
+		gtk_tree_path_free (path);
+	}
+	
+	if (view->priv->scroll_id == 0)
+	{
+		view->priv->scroll_id = 
+			g_timeout_add (150, 
+				       scroll_timeout, 
+				       GTK_TREE_VIEW (view));
+	}
+
+	gdk_drag_status (context, action, time);
 
 	return TRUE;
 }
 
-static gboolean
+static void
+drag_leave_cb (GtkWidget *widget,
+	       GdkDragContext *context,
+	       guint32 time,
+	       EphyNodeView *view)
+{
+	clear_drag_dest_row (view);
+
+	free_drag_data (view);
+
+	remove_scroll_timeout (view);
+}
+
+static void
 drag_data_received_cb (GtkWidget *widget,
-                       GdkDragContext *context,
-                       gint x,
-                       gint y,
-	               GtkSelectionData *selection_data,
-                       guint info,
-                       guint time,
-	               EphyNodeView *view)
+		       GdkDragContext *context,
+		       int x,
+		       int y,
+		       GtkSelectionData *selection_data,
+		       guint info,
+		       guint32 time,
+		       EphyNodeView *view)
 {
-	GtkTreePath *path = NULL;
 	GtkTreeViewDropPosition pos;
 
-	g_signal_stop_emission_by_name (widget, "drag_data_received");
+	/* x and y here are valid only on drop ! */
 
-	if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),
-					       x, y, &path, &pos))
+	if (selection_data->length <= 0 || selection_data->data == NULL)
+	{
+		return;
+	}	
+
+	if (!view->priv->have_drag_data)
+	{
+		view->priv->have_drag_data = TRUE;
+		view->priv->drag_data = 
+			gtk_selection_data_copy (selection_data);
+	}
+
+	if (view->priv->drop_occurred)
 	{
 		EphyNode *node;
 		GList *uris;
+		gboolean success = FALSE;
+		GtkTreePath *path;
+
+		gtk_tree_view_get_dest_row_at_pos
+			(GTK_TREE_VIEW (widget), x, y, &path, &pos);
+
+		g_return_if_fail (path != NULL);
 
 		node = get_node_from_path (view, path);
 
 		uris = gnome_vfs_uri_list_parse (selection_data->data);
-		g_signal_emit (G_OBJECT (view),
-			       ephy_node_view_signals[NODE_DROPPED], 0,
-			       node, uris);
-		gnome_vfs_uri_list_free (uris);
+	
+		if (uris != NULL)
+		{
+			/* FIXME fill success */
+			g_signal_emit (G_OBJECT (view),
+				       ephy_node_view_signals[NODE_DROPPED], 0,
+				       node, uris);
+			gnome_vfs_uri_list_free (uris);
+		}
 
-		gtk_tree_path_free (path);
+		view->priv->drop_occurred = FALSE;
+		free_drag_data (view);
+		gtk_drag_finish (context, success, FALSE, time);
+
+		if (path)
+		{
+			gtk_tree_path_free (path);
+		}
 	}
 
+
+	/* appease GtkTreeView by preventing its drag_data_receive
+	 * from being called */
+	g_signal_stop_emission_by_name (GTK_TREE_VIEW (view),
+					"drag_data_received");
+}
+
+static gboolean
+drag_drop_cb (GtkWidget *widget,
+	      GdkDragContext *context,
+	      int x, 
+	      int y,
+	      guint32 time,
+	      EphyNodeView *view)
+{
+	view->priv->drop_occurred = TRUE;
+
+	get_drag_data (view, context, time);
+	remove_scroll_timeout (view);
+	clear_drag_dest_row (view);
+	
 	return TRUE;
 }
 
@@ -334,9 +523,9 @@ ephy_node_view_enable_drag_dest (EphyNodeView *view,
 
 	treeview = GTK_WIDGET (view);
 
-	gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (treeview),
-					      types, n_types,
-					      GDK_ACTION_COPY);
+	gtk_drag_dest_set (GTK_WIDGET (treeview),
+			   0, types, n_types,
+			   GDK_ACTION_COPY);
 	view->priv->drag_targets = gtk_target_list_new (types, n_types);
 
 	g_signal_connect (treeview, "drag_data_received",
@@ -345,6 +534,8 @@ ephy_node_view_enable_drag_dest (EphyNodeView *view,
 			  G_CALLBACK (drag_drop_cb), view);
 	g_signal_connect (treeview, "drag_motion",
 			  G_CALLBACK (drag_motion_cb), view);
+	g_signal_connect (treeview, "drag_leave",
+			  G_CALLBACK (drag_leave_cb), view);
 }
 
 static void
@@ -484,6 +675,170 @@ ephy_node_view_key_press_cb (GtkTreeView *treeview,
 	return FALSE;
 }
 
+static void
+selection_foreach (GtkTreeModel *model,
+		   GtkTreePath *path,
+		   GtkTreeIter *iter,
+		   gpointer data)
+{
+	GList **list;
+                                                                                                                              
+	list = (GList**)data;
+                                                                                                                              
+	*list = g_list_prepend (*list,
+				gtk_tree_row_reference_new (model, path));              
+}
+
+static GList *
+get_selection_refs (GtkTreeView *tree_view)
+{
+	GtkTreeSelection *selection;
+	GList *ref_list;
+                                                                                                                              
+	ref_list = NULL;
+                                                                                                                              
+	selection = gtk_tree_view_get_selection (tree_view);
+	gtk_tree_selection_selected_foreach (selection,
+					     selection_foreach,
+					     &ref_list);
+	ref_list = g_list_reverse (ref_list);
+                                                                                                                              
+	return ref_list;
+}
+                                                                                                                              
+static void
+ref_list_free (GList *ref_list)
+{
+        g_list_foreach (ref_list, (GFunc) gtk_tree_row_reference_free, NULL);
+        g_list_free (ref_list);
+}
+                                                                                                                              
+static void
+stop_drag_check (EphyNodeView *view)
+{
+        view->priv->drag_button = 0;
+}
+
+static gboolean
+button_event_modifies_selection (GdkEventButton *event)
+{
+	return (event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) != 0;
+}
+
+static void
+did_not_drag (EphyNodeView *view,
+              GdkEventButton *event)
+{
+        GtkTreeView *tree_view;
+        GtkTreeSelection *selection;
+        GtkTreePath *path;
+                                                                                                                              
+        tree_view = GTK_TREE_VIEW (view);
+        selection = gtk_tree_view_get_selection (tree_view);
+                                                                                                                              
+        if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y,
+                                           &path, NULL, NULL, NULL))
+	{
+                if((event->button == 1 || event->button == 2) &&
+                   gtk_tree_selection_path_is_selected (selection, path) &&
+                   !button_event_modifies_selection (event))
+		{
+                        gtk_tree_selection_unselect_all (selection);
+                        gtk_tree_selection_select_path (selection, path);
+                }
+                                                                                                                              
+                gtk_tree_path_free (path);
+        }                                                                                                              
+}
+
+static void
+drag_data_get_cb (GtkWidget *widget,
+                  GdkDragContext *context,
+                  GtkSelectionData *selection_data,
+                  guint info,
+                  guint time)
+{
+	GtkTreeView *tree_view;
+	GtkTreeModel *model;
+	GList *ref_list;
+                                                                                                                              
+	tree_view = GTK_TREE_VIEW (widget);
+                                                                                                                              
+	model = gtk_tree_view_get_model (tree_view);
+	g_return_if_fail (model != NULL);
+                                                                                                                              
+	ref_list = g_object_get_data (G_OBJECT (context), "drag-info");
+                                                                                                                              
+	if (ref_list == NULL)
+	{
+        	return;
+	}
+                                                                                                                              
+	if (EGG_IS_TREE_MULTI_DRAG_SOURCE (model))
+	{
+		egg_tree_multi_drag_source_drag_data_get (EGG_TREE_MULTI_DRAG_SOURCE (model),
+							  ref_list,
+							  selection_data);
+	}
+}
+
+static gboolean
+button_release_cb (GtkWidget *widget,
+                   GdkEventButton *event,
+                   EphyNodeView *view)
+{
+        if (event->button == view->priv->drag_button)
+	{
+		stop_drag_check (view);
+		if (!view->priv->drag_started)
+		{
+                        did_not_drag (view, event);
+                        return TRUE;
+                }
+        }
+        return FALSE;
+}
+
+static gboolean
+motion_notify_cb (GtkWidget *widget,
+		  GdkEventMotion *event,
+		  EphyNodeView *view)
+{
+        GdkDragContext *context;
+        GList *ref_list;
+                                                                                                      
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget)))
+	{
+		return FALSE;
+	}                                                                                                                      
+        if (view->priv->drag_button != 0)
+	{
+		if (gtk_drag_check_threshold (widget, view->priv->drag_x,
+                                              view->priv->drag_y, event->x,
+					      event->y))
+		{
+			context = gtk_drag_begin
+				(widget, view->priv->source_target_list,
+                                 GDK_ACTION_COPY,
+                                 view->priv->drag_button,
+                                 (GdkEvent*)event);
+
+			stop_drag_check (view);
+			view->priv->drag_started = TRUE;
+
+			ref_list = get_selection_refs (GTK_TREE_VIEW (widget));
+			g_object_set_data_full (G_OBJECT (context),
+						"drag-info",
+						ref_list,
+						(GDestroyNotify)ref_list_free);
+
+			gtk_drag_set_icon_default (context);
+		}
+	}
+
+	return TRUE;
+}
+
 static gboolean
 ephy_node_view_button_press_cb (GtkWidget *treeview,
 			        GdkEventButton *event,
@@ -491,8 +846,8 @@ ephy_node_view_button_press_cb (GtkWidget *treeview,
 {
 	GtkTreePath *path = NULL;
 	GtkTreeSelection *selection;
-
-	if (event->button != 3) return FALSE;
+	gboolean call_parent = TRUE, path_is_selected;
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 
 	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
 					   event->x,
@@ -500,19 +855,48 @@ ephy_node_view_button_press_cb (GtkWidget *treeview,
 					   &path,
 					   NULL, NULL, NULL))
 	{
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-		if (!gtk_tree_selection_path_is_selected (selection, path))
-		{
-			GTK_WIDGET_CLASS (parent_class)->button_press_event (treeview, event);
-		}
-		else if (!gtk_widget_is_focus (GTK_WIDGET (treeview)))
+		path_is_selected = gtk_tree_selection_path_is_selected (selection, path);
+
+		if (!gtk_widget_is_focus (GTK_WIDGET (treeview)))
 		{
 			gtk_widget_grab_focus (GTK_WIDGET (treeview));
 		}
 
-		g_signal_emit (G_OBJECT (view), ephy_node_view_signals[SHOW_POPUP], 0);
+		if (event->button == 3 && path_is_selected)
+		{
+			call_parent = FALSE;
+		}
+
+		if(!button_event_modifies_selection (event) &&
+                   event->button == 1 && path_is_selected &&
+		   gtk_tree_selection_count_selected_rows (selection) > 1)
+		{
+                        call_parent = FALSE;
+                }
+
+		if (call_parent)
+		{
+			GTK_WIDGET_CLASS (parent_class)->button_press_event (treeview, event);
+		}
+
+		if (event->button == 3)
+		{
+			g_signal_emit (G_OBJECT (view), ephy_node_view_signals[SHOW_POPUP], 0);
+		}
+		else if (event->button == 1)
+		{
+			view->priv->drag_started = FALSE;
+			view->priv->drag_button = event->button;
+	                view->priv->drag_x = event->x;
+	                view->priv->drag_y = event->y;
+	        }
+
 		gtk_tree_path_free (path);
 	}
+	else
+	{
+		gtk_tree_selection_unselect_all (selection);
+	}
 
 	return TRUE;
 }
@@ -660,6 +1044,38 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	gtk_tree_path_free (path);
 }
 
+static int
+compare_string_values (const GValue *a_value, const GValue *b_value)
+{
+        const char *str1, *str2;
+        int retval;
+                                                                                                                              
+        str1 = g_value_get_string (a_value);
+        str2 = g_value_get_string (b_value);
+                                                                                                                              
+        if (str1 == NULL)
+        {
+                retval = -1;
+        }
+        else if (str2 == NULL)
+        {
+                retval = 1;
+        }
+        else
+        {
+	        char *str_a;
+	        char *str_b;
+
+                str_a = g_utf8_casefold (str1, -1);
+                str_b = g_utf8_casefold (str2, -1);
+                retval = g_utf8_collate (str_a, str_b);
+                g_free (str_a);
+                g_free (str_b);
+        }
+                                                                                                                              
+        return retval;
+}
+
 static int
 ephy_node_view_sort_func (GtkTreeModel *model,
 			  GtkTreeIter *a,
@@ -681,20 +1097,14 @@ ephy_node_view_sort_func (GtkTreeModel *model,
 		GType type = gtk_tree_model_get_column_type (model, column);
 		GValue a_value = {0, };
 		GValue b_value = {0, };
-		gchar *stra, *strb;
+
 		gtk_tree_model_get_value (model, a, column, &a_value);
 		gtk_tree_model_get_value (model, b, column, &b_value);
 
 		switch (G_TYPE_FUNDAMENTAL (type))
 		{
 		case G_TYPE_STRING:
-			/* FIXME: this is horribly inefficient */
-			stra = g_utf8_casefold (g_value_get_string (&a_value), -1);
-			strb = g_utf8_casefold (g_value_get_string (&b_value), -1);
-			g_return_val_if_fail (stra != NULL && strb != NULL, 0);
-			retval = g_utf8_collate (stra, strb);
-			g_free (stra);
-			g_free (strb);
+			retval = compare_string_values (&a_value, &b_value);
 			break;
 		case G_TYPE_INT:
 			if (g_value_get_int (&a_value) < g_value_get_int (&b_value))
@@ -902,6 +1312,7 @@ ephy_node_view_init (EphyNodeView *view)
 	view->priv->editable_renderer = NULL;
 	view->priv->editing = TRUE;
 	view->priv->searchable_data_column = -1;
+	view->priv->source_target_list = NULL;
 
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), FALSE);
 }
@@ -963,7 +1374,8 @@ ephy_node_view_remove (EphyNodeView *view)
 	 */
 
 	list = ephy_node_view_get_selection (view);
-	g_return_if_fail (list != NULL);
+	if (list == NULL) return;
+
 	node = g_list_first (list)->data;
 	ephy_tree_model_node_iter_from_node (EPHY_TREE_MODEL_NODE (view->priv->nodemodel),
 					     node, &iter);
@@ -1050,15 +1462,27 @@ ephy_node_view_enable_drag_source (EphyNodeView *view,
 
 	treeview = GTK_WIDGET (view);
 
-	egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (treeview));
-
-	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (treeview),
-						GDK_BUTTON1_MASK,
-						types, n_types,
-						GDK_ACTION_COPY);
+	view->priv->source_target_list =
+		gtk_target_list_new (types, n_types);
 
 	ephy_tree_model_sort_set_column_id (EPHY_TREE_MODEL_SORT (view->priv->sortmodel),
 					    column);
+
+	g_signal_connect_object (G_OBJECT (view),
+			         "button_release_event",
+			         G_CALLBACK (button_release_cb),
+			         view,
+				 0);
+	g_signal_connect_object (G_OBJECT (view),
+			         "motion_notify_event",
+			         G_CALLBACK (motion_notify_cb),
+			         view,
+				 0);
+	g_signal_connect_object (G_OBJECT (view),
+			         "drag_data_get",
+			         G_CALLBACK (drag_data_get_cb),
+			         view,
+				 0);
 }
 
 void
