@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: ephy-bookmarks-editor.c,v 1.97.2.4 2003/10/17 12:06:20 chpe Exp $
+ * $Id: ephy-bookmarks-editor.c,v 1.97.2.5 2003/11/08 18:27:06 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -803,16 +803,13 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	if (GTK_IS_EDITABLE (focus_widget))
 	{
 		gboolean has_selection;
-		gboolean clipboard_contains_text;
 
 		has_selection = gtk_editable_get_selection_bounds
 			(GTK_EDITABLE (focus_widget), NULL, NULL);
-		clipboard_contains_text = gtk_clipboard_wait_is_text_available
-			(gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
 
 		cut = has_selection;
 		copy = has_selection;
-		paste = clipboard_contains_text;
+		paste = TRUE;
 		select_all = TRUE;
 	}
 	else
