@@ -209,11 +209,10 @@ ephy_main_start (gpointer data)
 		gtk_dialog_run (GTK_DIALOG (dialog));
 
 	}
-	/* FIXME ephy --server doesnt work when not first istance */
 	/* Server mode */
 	else if (ephy_server_mode)
 	{
-		g_object_ref (G_OBJECT(ephy_shell));
+		GNOME_EphyAutomation_setServerMode (gaserver, TRUE, &corba_env);
 	}
 	/* Launch the bookmarks editor */
 	else if (open_as_bookmarks_editor)
@@ -239,20 +238,7 @@ ephy_main_start (gpointer data)
 			(gaserver, quit_option, &corba_env);
 	}
 	/* provided with urls? */
-	else if (n_urls == 0 &&
-		 !open_as_nautilus_view)
-	{
-		/* no, open a default window */
-		GNOME_EphyAutomation_loadurl
-			(gaserver, "",
-			 open_fullscreen,
-			 open_in_existing,
-			 open_in_new_window,
-			 open_in_new_tab,
-			 !noraise,
-			 &corba_env);
-	}
-	else
+	else if (!open_as_nautilus_view)
 	{
 		/* open all of the urls */
 		for (i = 0; i < n_urls; i++)
@@ -268,8 +254,11 @@ ephy_main_start (gpointer data)
 		}
 	}
 
-	/* Unref so it will exit if no more used */
-	if (first_instance)
+	/* Unref so it will exit if no more used
+	 * If started with --nautilus-view, only exit when the
+	 * last view has been destroyed.
+	 */
+	if (first_instance && !open_as_nautilus_view)
 	{
 		g_object_unref (G_OBJECT(ephy_shell));
 	}
@@ -308,6 +297,18 @@ ephy_main_automation_init (void)
 	}
 }
 
+/* dummy argument to open a default window */
+static gint
+ephy_main_dummy_url_argument (gchar ***urls)
+{
+       *urls = g_new0 (gchar *, 2);
+
+       (*urls)[0] = g_strdup ("");
+       (*urls)[1] = NULL;
+
+       return 1;
+}
+
 /**
  * translate_url_arguments: gather URL arguments and expand them fully
  * with realpath if they're filenames
@@ -322,8 +323,7 @@ ephy_main_translate_url_arguments (poptContext context, gchar ***urls)
         /* any context remaining? */
         if (context == NULL)
         {
-                *urls = NULL;
-                return 0;
+		return ephy_main_dummy_url_argument (urls);
         }
 
         /* get the args and check */
@@ -331,8 +331,7 @@ ephy_main_translate_url_arguments (poptContext context, gchar ***urls)
         if (args == NULL)
         {
                 poptFreeContext (context);
-                *urls = NULL;
-                return 0;
+		return ephy_main_dummy_url_argument (urls);
         }
 
         /* count args */
