@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.54.2.1 2003/09/24 20:39:52 chpe Exp $
+ *  $Id: ephy-shell.c,v 1.54.2.2 2003/10/17 09:51:34 markmc Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -491,23 +491,45 @@ ephy_shell_new_tab (EphyShell *shell,
 
 #ifdef ENABLE_NAUTILUS_VIEW
 
+static void
+ephy_nautilus_view_all_controls_dead (void)
+{
+        if (!bonobo_control_life_get_count ())
+	{
+		g_object_unref (G_OBJECT (ephy_shell));
+	}
+}
+
 static void
 ephy_nautilus_view_init_factory (EphyShell *gs)
 {
 	BonoboGenericFactory *ephy_nautilusview_factory;
+	
 	ephy_nautilusview_factory = bonobo_generic_factory_new
 		(EPHY_NAUTILUS_VIEW_OAFIID,
 		 (BonoboFactoryCallback) ephy_nautilus_view_new, gs);
 	if (!BONOBO_IS_GENERIC_FACTORY (ephy_nautilusview_factory))
+	{
 		g_warning ("Couldn't create the factory!");
+		return;
+	}
 
+	bonobo_control_life_set_callback (ephy_nautilus_view_all_controls_dead);
 }
 
 static BonoboObject *
 ephy_nautilus_view_new (BonoboGenericFactory *factory, const char *id,
 			  EphyShell *gs)
 {
-	return ephy_nautilus_view_new_component (gs);
+	EphyNautilusView *view;
+
+	view = EPHY_NAUTILUS_VIEW (
+		ephy_nautilus_view_new_component (gs));
+
+	bonobo_control_life_instrument (
+		nautilus_view_get_bonobo_control (NAUTILUS_VIEW (view)));
+
+	return BONOBO_OBJECT (view);
 }
 
 #endif
