@@ -22,6 +22,7 @@
 #include "ephy-embed.h"
 #include "ephy-dialog.h"
 #include "ephy-notebook.h"
+
 #include <glib-object.h>
 #include <glib.h>
 #include <gtk/gtkwindow.h>
@@ -62,21 +63,6 @@ typedef enum
 
 typedef enum
 {
-	TabsControl,
-	NavControl,
-	FindControl,
-	ZoomControl,
-	CharsetsControl,
-	TitleControl,
-	LocationControl,
-	FaviconControl,
-	StatusbarSecurityControl,
-	StatusbarMessageControl,
-	StatusbarProgressControl,
-	SpinnerControl,
-	WindowVisibilityControl,
-	BMAndHistoryControl,
-	TabsAppeareanceControl,
 	FavoritesControl
 } ControlID;
 
@@ -96,6 +82,7 @@ GtkWidget	 *ephy_window_get_notebook	  (EphyWindow *window);
 
 void		  ephy_window_add_tab		  (EphyWindow *window,
 						   EphyTab *tab,
+						   gint position,
 						   gboolean jump_to);
 
 void		  ephy_window_remove_tab	  (EphyWindow *window,
@@ -108,15 +95,13 @@ void		  ephy_window_load_url		  (EphyWindow *window,
 						   const char *url);
 
 void		  ephy_window_set_zoom		  (EphyWindow *window,
-						   gint zoom);
+						   float zoom);
 
 void		  ephy_window_activate_location	  (EphyWindow *window);
 
 void		  ephy_window_update_control	  (EphyWindow *window,
 						   ControlID control);
 
-void		  ephy_window_update_all_controls (EphyWindow *window);
-
 EphyTab		 *ephy_window_get_active_tab	  (EphyWindow *window);
 
 EphyEmbed	 *ephy_window_get_active_embed	  (EphyWindow *window);
@@ -127,9 +112,9 @@ Toolbar		 *ephy_window_get_toolbar	  (EphyWindow *window);
 
 /* Dialogs */
 
-EphyDialog       *ephy_window_get_find_dialog	  (EphyWindow *window);
+void		  ephy_window_find		  (EphyWindow *window);
 
-void		  ephy_window_show_history	  (EphyWindow *window);
+void		  ephy_window_print		  (EphyWindow *window);
 
 G_END_DECLS
 
