@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.76 2003/08/18 22:42:39 marco Exp $
+ *  $Id: window-commands.c,v 1.76.2.2 2003/09/23 22:23:22 chpe Exp $
  */
 
 #include <config.h>
@@ -210,18 +210,21 @@ window_cmd_view_reload (EggAction *action,
 	g_return_if_fail (embed != NULL);
 
 	event = gtk_get_current_event ();
-	type = event->type;
-
-	if (type == GDK_BUTTON_RELEASE)
+	if (event)
 	{
-		state = event->button.state; 
-	}
-	else if (type == GDK_KEY_RELEASE)
-	{
-		state = event->key.state;
-	}
+		type = event->type;
 
-	gdk_event_free (event);
+		if (type == GDK_BUTTON_RELEASE)
+		{
+			state = event->button.state; 
+		}
+		else if (type == GDK_KEY_RELEASE)
+		{
+			state = event->key.state;
+		}
+
+		gdk_event_free (event);
+	}
 
 	if (state & GDK_SHIFT_MASK)
 	{
@@ -318,10 +321,7 @@ void
 window_cmd_file_open (EggAction *action,
 		      EphyWindow *window)
 {
-	gchar *dir, *retDir;
-        gchar *file;
-        GnomeVFSURI *uri;
-	GtkWidget *wmain;
+	char *dir, *ret_dir, *file;
 	EphyEmbedShell *embed_shell;
 	gresult result;
 	EphyEmbedSingle *single;
@@ -331,35 +331,37 @@ window_cmd_file_open (EggAction *action,
 
 	embed_shell = EPHY_EMBED_SHELL (ephy_shell);
 
-	wmain = GTK_WIDGET (window);
-	g_return_if_fail (wmain != NULL);
-
         dir = eel_gconf_get_string (CONF_STATE_OPEN_DIR);
 
 	result = ephy_embed_single_show_file_picker
-		(single, wmain,
+		(single, GTK_WIDGET (window),
 		 _("Open"),
                  dir, NULL, modeOpen,
                  &file, NULL, NULL);
 
-	uri = gnome_vfs_uri_new (file);
-	if (uri)
+	/* persist directory choice */
+	/* Fix for bug 122780:
+	 * if the user selected a directory, or aborted with no filename typed,
+	 * g_path_get_dirname and gnome_vfs_uri_extract_dirname strip the last
+	 * path component, so test if the returned file is actually a directory.
+	 */
+	if (g_file_test (file, G_FILE_TEST_IS_DIR))
 	{
-		retDir = gnome_vfs_uri_extract_dirname (uri);
-
-		/* set default open dir */
-		eel_gconf_set_string (CONF_STATE_OPEN_DIR,
-				      retDir);
-
-		g_free (retDir);
-		gnome_vfs_uri_unref (uri);
-
-		if (result == G_OK)
-		{
-			ephy_window_load_url(window, file);
-	        }
+		ret_dir = g_strdup (file);
+	}
+	else
+	{
+		ret_dir = g_path_get_dirname (file);
 	}
 
+	eel_gconf_set_string (CONF_STATE_OPEN_DIR, ret_dir);
+
+	if (result == G_OK)
+	{
+		ephy_window_load_url(window, file);
+        }
+
+	g_free (ret_dir);
 	g_free (file);
         g_free (dir);
 }
